import Globals

import ZUBB_files, ZUBB_methods
from ZUBBMessage import ZUBBMessage

from AccessControl import ClassSecurityInfo
from Acquisition import aq_base 
from Products.ZCatalog import ZCatalog
from Globals import HTMLFile
from Products.PythonScripts.standard import html_quote

emoticons_list =     {'happy.gif':    (':-)', ':)', '=)')
                    , 'unhappy.gif':  (':-(', ':(', '=(', ":'(")
                    , 'blah.gif':     (':-|', ':|', '=|', ':-I', ':I', '=|')
                    , 'funny.gif':    (':-D', ':D', '=D')
                    , 'confused.gif': (':-S', ':S', '=S', '=s', ':-s', ':s', ':-?')
                    , 'ooh.gif':      (':-O', ':O', '=O', '=o', ':-o', ':o')
                    , 'wink.gif':     (';-)', ';)', ';D')
                    , 'poke.gif':     (':-P', ':P', '=P', '=p', '=b', ':-p', ':p', ':-b', ':b')
                    }

catalog_indexes =   ( ('author', 'FieldIndex')
                    , ('all_text', 'ZCTextIndex')
                    , ('email', 'FieldIndex')
                    , ('in_reply_to', 'FieldIndex')
                    , ('last_post_date', 'FieldIndex')
                    , ('last_post_author', 'FieldIndex')
                    , ('posted_date', 'FieldIndex')
                    , ('priority', 'FieldIndex')
                    )

catalog_metadata =   ('author'
                    , 'email'
                    , 'id'
                    , 'in_reply_to'
                    , 'last_post_date'
                    , 'last_post_author'
                    , 'posted_date'
                    , 'priority'
                    , 'title'
                    )

# this is used for passing record fields
class _record: pass

manage_addZUBBFolderForm = HTMLFile('dtml/ZUBBFolder_addForm', globals())

def manage_addZUBBFolder(self, id, title, REQUEST=None):
    """Add a ZUBBFolder object"""

    folder = ZUBBFolder(id, title)
    self._setObject(id, folder)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)


class ZUBBFolder(ZCatalog.ZCatalog):
    """ZUBBFolder object"""

    meta_type = 'ZUBB Folder'

    security = ClassSecurityInfo()

    security.setPermissionDefault('Add ZUBB Messages', ('Anonymous','Manager'))
    security.setPermissionDefault('Delete ZUBB Folders', ('Manager',))
    security.setPermissionDefault('Manage ZUBB Folders', ('Manager',))

    security.declareProtected('View management screens', 'manage_catalogOptions')
    manage_catalogOptions = Globals.HTMLFile('dtml/catalogOptions', globals())

    security.declareProtected('Manage ZUBB Folders', 'manage_ZUBBOptions')
    manage_ZUBBOptions = Globals.HTMLFile('dtml/ZUBBOptions', globals())

    manage_options=(
        {'label': 'Contents',
         'action': 'manage_main'},
        
        {'label': 'View',
         'action': ''},
        
        {'label': 'ZUBB Options',
         'action': 'manage_ZUBBOptions'},
        
        {'label': 'Catalog Options',
         'action': 'manage_catalogOptions'},
        
        {'label': 'Security',
         'action': 'manage_access'},

        {'label': 'Undo',
         'action': 'manage_UndoForm'}
        )

    security.declarePublic('get_batch'
                         , 'text_limit'
                         , 'text_format'
                         , 'querystring_change'
                         , 'date_format'
                         , 'get_subfolders'
                    )
    get_batch = ZUBB_methods.get_batch
    text_limit = ZUBB_methods.text_limit
    text_format = ZUBB_methods.text_format
    querystring_change = ZUBB_methods.querystring_change
    date_format = ZUBB_methods.date_format
    get_subfolders = ZUBB_methods.get_subfolders

    def __init__(self, id, title=''):
        ZCatalog.ZCatalog.__init__(self, id, title)   # Initialise ZCatalog
        self._indexes = 0
        self._defaultfiles = 0
        base = aq_base(self)
        for column_name in catalog_metadata:
            self.addColumn(column_name)
        # indexes are set up in manage_afterAdd

    security.declareProtected('Manage ZUBB Folders', 'manage_addDefaultFiles')
    manage_addDefaultFiles = ZUBB_files.manage_addDefaultFiles

    security.declareProtected('Add ZUBB Messages', 'manage_addZUBBMessage')          
    def manage_addZUBBMessage(self, id, title, body, author, email, in_reply_to='', priority=0, REQUEST=None):
        """Add a ZUBBMessage to this ZUBBFolder"""
        title = title.strip()
        body.strip()
        # only add message if there is a title or some body text
        if (title!='' and title!='RE:') or body!='':
            if not id:
                id = self.generate_id()
            message = ZUBBMessage(id, title, body, author, email, in_reply_to, priority)
            self._setObject(id, message)
        else:
            id = None
        if REQUEST is not None:
            if not id:
                # didn't add message
                REQUEST.RESPONSE.redirect(REQUEST.URL1)
            else:
                # redirect
                if REQUEST.form.has_key('goto'):
                    REQUEST.RESPONSE.redirect('%s' % REQUEST.form.goto)
                else:
                    REQUEST.RESPONSE.redirect('%s' % id)

    def manage_afterAdd(self, item, container):
        self.no_orphan_fixing = 0   # allow thread flattening
        ZCatalog.ZCatalog.manage_afterAdd(self, item, container)
        # set up the indexes
        if not self._indexes:
            casenormalizer = _record()
            stopwords = _record()
            recordsplitter = _record()
            casenormalizer.group = 'Case Normalizer'
            casenormalizer.name = 'Case Normalizer'
            stopwords.group = 'Stop Words'
            stopwords.name = ' Don\'t remove stop words'
            recordsplitter.group = 'Word Splitter'
            recordsplitter.name = 'Whitespace splitter'
            records = [casenormalizer, stopwords, recordsplitter]
            self.manage_addProduct['ZCTextIndex'].manage_addLexicon('lexicon', 'Stores message text for searching', elements = records)
            extra = _record()
            extra.index_type = 'Okapi BM25 Rank'
            extra.lexicon_id = 'lexicon'
            for index_name, index_type in catalog_indexes:
                if index_type == 'ZCTextIndex':
                    extra.doc_attr = index_name
                    self.addIndex(index_name, index_type, extra)
                else:
                    self.addIndex(index_name, index_type)
            self._indexes = 1
        # add the default files unless our parent is also a ZUBB folder
        if not self._defaultfiles:
            if self.aq_parent.meta_type != 'ZUBB Folder':
                self.manage_addDefaultFiles()
            self._defaultfiles = 1

    security.declareProtected('Delete ZUBB Messages', 'manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        self.no_orphan_fixing = 1   # so that we wont flatten our thread if this is a move / rename
        ZCatalog.ZCatalog.manage_beforeDelete(self, item, container)

    security.declareProtected('Delete ZUBB Messages', 'manage_deleteZUBBMessage')
    def manage_deleteZUBBMessage(self, id, REQUEST=None):
        """Delete a ZUBBMessage from this ZUBBFolder"""
        if hasattr(self, id):
            if isinstance(self[id], ZUBBMessage):
                self.manage_delObjects([id])
        if REQUEST:
            REQUEST.RESPONSE.redirect('%s/manage_main' % REQUEST.URL1)

    security.declareProtected('Manage ZUBB Folders', 'manage_editZUBBFolder')
    def manage_editZUBBFolder(self, title, REQUEST=None):
        """Set properties of ZUBBFolder.  Presently just title."""
        self.title = title
        if REQUEST:
            return self.manage_ZUBBOptions(self, REQUEST, manage_tabs_message = "ZUBB Folder title changed.")

    security.declareProtected('Manage ZUBB Folders', 'manage_recatalog')
    def manage_recatalog(self, REQUEST = None):
        """ Recatalog all messages in the catalog """
        self.manage_catalogClear()
        results = self.objectItems(['ZUBB Message'])
        for id, ob in results:
            ob._catalog()
        if REQUEST:
            return self.manage_main(self, REQUEST, manage_tabs_message = "ZUBB Folder Recatalogued.")

    security.declareProtected('Manage ZUBB Folders', 'manage_regenerateDefaultMethods')
    manage_regenerateDefaultMethods = HTMLFile('dtml/regenerateDefaultMethods', globals())

    security.declareProtected('Manage ZUBB Folders', 'manage_saveOptions')
    def manage_saveOptions(self, date_structure, primary_view, secondary_view, emoticons_directory, date_structure_custom = '', emoticons_directory_custom = '', date_friendly = 1, inherit = 1, skin_id = '', search_method = 0, REQUEST = None):
        """ Save ZUBB Folder options """
        if date_structure_custom:
            date_structure = date_structure_custom
        if date_structure:
            self._option_date_structure = date_structure
        if emoticons_directory_custom:
            emoticons_directory = emoticons_directory_custom
        if emoticons_directory:
            self._option_emoticons_directory = emoticons_directory
        self._option_primary_view = primary_view
        if secondary_view == primary_view:
            secondary_view = 0
        self._option_secondary_view = secondary_view
        self._option_date_friendly = date_friendly
        self._option_inherit = inherit
        try:
            self._option_skin = self['skins'][skin_id]
        except:
            if not hasattr(self, 'skins'):
                zubb_files.addFolder(self, 'skins')
            self._option_skin = self['skins']
        self._option_search_method = search_method
        self._p_changed = 1   # updates the folder object properly
        if REQUEST:
            return self.manage_main(self, REQUEST, manage_tabs_message = "ZUBB Folder options saved.")

    security.declareProtected('Delete ZUBB Messages', 'do_delete')
    def do_delete(self, REQUEST):
        """Delete a ZUBB Message"""
        id = REQUEST.form.get('delete')
        message = ''
        if id:
            if hasattr(self, id):
                try:
                    self.manage_deleteZUBBMessage(id)
                    message = 'Post deleted'
                except:
                    message = 'Could not delete: an error occured while trying to delete post'
            else:
                message = 'Could not delete: post not found'
        querystring = self.querystring_change(REQUEST.QUERY_STRING, ('delete',('message', message)))
        if REQUEST:
            REQUEST.RESPONSE.redirect('%s%s' % (REQUEST.URL1, querystring))
            
    security.declareProtected('Add ZUBB Messages', 'do_post')
    def do_post(self, REQUEST):
        """Post a ZUBB Message"""
        f = REQUEST.form
        title = f.get('title', '')
        body = f.get('body', '')
        author = f.get('author', '')
        email = f.get('email', '')
        in_reply_to = f.get('in_reply_to', '')
        priority = f.get('priority', 0)
        # set cookies
        resp = REQUEST.RESPONSE
        if author:
            resp.setCookie('zubbAuthor', author, path='/')
        else:
            resp.expireCookie('zubbAuthor')
        if email:
            resp.setCookie('zubbEmail', email, path='/')
        else:
            resp.expireCookie('zubbEmail')
        self.manage_addZUBBMessage('', title, body, author, email, in_reply_to, priority, REQUEST)

    security.declarePublic('emoticon_html')
    def emoticon_html(self, file, emoticon, directory = ''):
        """ Return html for the emoticon image """
        emoticon = html_quote(emoticon)
        if not directory:
            directory = getattr(self.zubb_options(), 'emoticons_directory', '/misc_/ZUBB')
        return '<img src="%s/%s" alt="%s" />' % (directory, file, emoticon)

    security.declarePublic('forum_url')
    def forum_url(self):
        return '/%s' % self.absolute_url(1)
    
    security.declarePublic('generate_id')
    def generate_id(self):
        """Find an available ID"""
        fmt = "%08d"
        id = 0
        modifier = 1
        direction = -1
        increment = 1
        hasit = hasattr(self.aq_explicit, fmt % id)
        while modifier != 1 or hasit:
            # check direction
            if hasit:
                direction = 1
            else:
                if increment:
                    increment = 0
                direction = -1
            # change modifier
            if increment:
                modifier = modifier * 2
            else:
                if modifier > 1:
                    modifier = modifier / 2
            # change id by modifier
            id = id + (direction * modifier)
            hasit = hasattr(self.aq_explicit, fmt % id)
        return fmt % id

    security.declarePublic('posts', 'children')
    def posts(self):
        """ Return the top-level posts - most recently posted into first """
        posts = []
        results = self.searchResults(in_reply_to=[''], sort_on='last_post_date', sort_order='reverse', priority=1)
        results += self.searchResults(in_reply_to=[''], sort_on='last_post_date', sort_order='reverse', priority=0)
        posts = map(lambda x: x.getObject(), results)
        return posts
    children = posts

    security.declarePublic('search_messages')
    def search_messages(self, words, author = '', threads_only = None, weighted = None):
        """ Returns all of the messages that contain the words specified """
        # search for partial words
        if words:
            words_list = words.split(' ')
            words_list = map(lambda x: x and '%s*' % x, words_list)
            words = ' '.join(words_list)
        # search for title and author
        if weighted == None:
            directory = getattr(self.zubb_options(), 'search_method', 0)
        if weighted == 1:
            # return results by weight
            results = self.searchResults(all_text = words, author = author)
        else:
            # return results by most recent
            results = self.searchResults(all_text = words, author = author, sort_on='posted_date', sort_order='reverse')
        if threads_only:
            # take only the tops of threads where the word occured and avoid double ups
            newresults = []
            for result in results:
                if result.in_reply_to:
                    newresult = result.thread_top
                else:
                    newresult = result
                if newresult not in newresults:
                    newresults.append(newresult)
            results = newresults
        return results

    security.declarePublic('zubb_emoticons')
    def zubb_emoticons(self):
        return emoticons_list
    
    security.declarePublic('zubb_options')
    def zubb_options(self):
        """ Return ZUBB Folder options (inherit if necessary) """
        options = {}
        inherit = getattr(self, '_option_inherit', 1)
        if inherit:
            if hasattr(self.aq_parent, 'zubb_options'):
                ancestor_options = self.aq_parent.zubb_options()
                if type(ancestor_options) == type(options):
                    options = ancestor_options
        options['inherit'] = inherit
        if not options.has_key('emoticons_directory'):
            options['emoticons_directory'] = getattr(self, '_option_emoticons_directory', '/misc_/ZUBB')
        if not options.has_key('date_structure'):
            options['date_structure'] = getattr(self, '_option_date_structure', '%d&nbsp;%b&nbsp;%Y')
        if not options.has_key('date_friendly'):
            options['date_friendly'] = getattr(self, '_option_date_friendly', 1)
        if not options.has_key('primary_view'):
            options['primary_view'] = getattr(self, '_option_primary_view', 1)
        if not options.has_key('secondary_view'):
            options['secondary_view'] = getattr(self, '_option_secondary_view', 2)
        if not options.has_key('skin'):
            if not hasattr(self, 'skins'):
                zubb_files.addFolder(self, 'skins')
            options['skin'] = getattr(self, '_option_skin', self['skins'])
        if not options.has_key('search_method'):
            options['search_method'] = getattr(self, '_option_search_method', 0)
        return options


# This gets the security assertions going
Globals.InitializeClass(ZUBBFolder)
